from ewoksorange.setuptools import setup

if __name__ == "__main__":
    setup(
        __file__,
        name="ewoks-example-2-addon",
        version="0.0.1a",
        description="Add-on containing Ewoks example widgets",
    )
